@@ -47,32 +47,67 @@
 #'                                                             rather than the individual level, which is not recommended for panel data.
 #'                                     \item \strong{seed}: Numeric scalar (integer). Random number generator seed to generate the bootstrap samples.
 #'                                                          Only used if \code{samples} is \code{NA}. Default is 24.
+#'                                     \item \strong{calledByEstimate}: Logical. TRUE if \code{apollo_bootstrap} is called by \code{apollo_estimate}. FALSE by default.
+#'                                     \item \strong{recycle}: Logical. If TRUE, the function will look for old output files and append new repetitions to them. If FALSE, output files will be overwritten.
 #'                                   }
-#' @return Covariance matrix of the \code{nRep} sets of estimated parameters. Also written to file.
+#' @return List with three elements.
+#'         \itemize{
+#'           \item \code{estimates}: Matrix containing the parameter estimates for each repetition. As many rows as repetitions and as many columns as parameters.
+#'           \item \code{varcov}: Covariance matrix of the estimated parameters across the repetitions.
+#'           \item \code{LL}: Vector of final loglikelihoods of each repetition.
+#'         }
+#'         This function also writes three output files to the working directory, with the following names ('x' represents the model name):
+#'         \itemize{
+#'           \item \strong{x_bootstrap_params.csv}: Table containing the parameter estimates, loglikelihood, and number of observations for each repetition.
+#'           \item \strong{x_bootstrap_samples.csv}: Table containing the description of the sample used in each repetition. Same format than \code{bootstrap_settings$samples}.
+#'           \item \strong{x_bootstrap_vcov}: Table containing the covariance matrix of estimated parameters across the repetitions.
+#'         }
 #' @export
 #' @importFrom maxLik maxLik
 #' @importFrom stats cov
 apollo_bootstrap <- function(apollo_beta, apollo_fixed,
-                               apollo_probabilities, apollo_inputs,
-                               estimate_settings=list(estimationRoutine="bfgs",
-                                                      maxIterations=200,
-                                                      writeIter=FALSE,
-                                                      hessianRoutine="none",
-                                                      printLevel=2L,
-                                                      silent=FALSE),
-                               bootstrap_settings=list(nRep=30,
-                                                       samples=NA,
-                                                       seed=24)){
+                             apollo_probabilities, apollo_inputs,
+                             estimate_settings=list(estimationRoutine="bfgs",
+                                                    maxIterations=200,
+                                                    writeIter=FALSE,
+                                                    hessianRoutine="none",
+                                                    printLevel=2L,
+                                                    silent=FALSE,
+                                                    maxLik_settings=list()),
+                             bootstrap_settings=list(nRep=30,
+                                                     samples=NA,
+                                                     seed=24,
+                                                     calledByEstimate=FALSE,
+                                                     recycle=TRUE)){
   
   ### Set missing settings to default values
-  default <- list(estimationRoutine="bfgs", maxIterations=200, writeIter=TRUE, hessianRoutine="none", printLevel=2L, silent=FALSE)
+  if(is.list(estimate_settings) && !is.null(estimate_settings$maxLik_settings)){
+    # Copy values in maxLik_settings into estimate_settings${printLevel, iterlim}
+    if(!is.null(estimate_settings$maxLik_settings$printLevel)) estimate_settings$printLevel <- estimate_settings$maxLik_settings$printLevel
+    if(!is.null(estimate_settings$maxLik_settings$iterlim)) estimate_settings$maxIterations <- estimate_settings$maxLik_settings$iterlim
+  }
+  default <- list(estimationRoutine="bfgs", maxIterations=200, writeIter=FALSE, 
+                  hessianRoutine="none", printLevel=2L, silent=FALSE, maxLik_settings=list())
   tmp <- names(default)[!(names(default) %in% names(estimate_settings))] # options missing in estimate_settings
   for(i in tmp) estimate_settings[[i]] <- default[[i]]
-  default <- list(nRep=30, samples=NA, seed=24)
-  tmp <- names(default)[!(names(default) %in% names(bootstrap_settings))] # options missing in estimate_settings
+  default <- list(nRep=30, samples=NA, seed=24, calledByEstimate=FALSE, recycle=TRUE)
+  tmp <- names(default)[!(names(default) %in% names(bootstrap_settings))] # options missing in bootstrap_settings
   for(i in tmp) bootstrap_settings[[i]] <- default[[i]]
   rm(tmp)
   
+  ### Write original apollo_inputs to disk before changing it, and make sure to restore it before finishing
+  if(!bootstrap_settings$calledByEstimate){
+    saveRDS(apollo_inputs, file=paste0(tempdir(), "\\", apollo_inputs$apollo_control$modelName,"_bootstrap"))
+    on.exit({
+      tmp <- paste0(tempdir(), "\\", apollo_inputs$apollo_control$modelName,"_bootstrap")
+      apollo_inputs <- tryCatch(readRDS(tmp), error=function(e) NULL)
+      tmp2 <- globalenv()
+      if(!is.null(apollo_inputs) & exists('apollo_inputs', envir=tmp2)) assign('apollo_inputs', apollo_inputs, envir=tmp2)
+      if(file.exists(tmp)) file.remove(tmp)
+      rm(tmp)
+    })
+  }
+  
   ### Extract values from apollo_inputs
   database         <- apollo_inputs$database
   apollo_inputs$apollo_control$noDiagnostics = TRUE
@@ -94,6 +129,8 @@ apollo_bootstrap <- function(apollo_beta, apollo_fixed,
   samples           <- bootstrap_settings$samples
   seed              <- bootstrap_settings$seed
   silent            <- estimate_settings$silent
+  calledByEstimate  <- bootstrap_settings$calledByEstimate
+  recycle           <- bootstrap_settings$recycle
   
   ### Validate arguments
   apollo_checkArguments(apollo_probabilities,apollo_randCoeff,apollo_lcPars)
@@ -116,74 +153,67 @@ apollo_bootstrap <- function(apollo_beta, apollo_fixed,
     if(ncol(samples)<2) stop("The 'samples' matrix must have at least two columns.")
   }
   
-  ### Separate theta into variable and fixed part
-  theta_var_val <- apollo_beta[!(names(apollo_beta) %in% apollo_fixed)]
-  theta_fix_val <- apollo_beta[apollo_fixed]
-  
   ### Start clock
   starttime <- Sys.time()
   
   ### Initial report
   if(anyNA(samples)){
-    indivs  <- sort(unique(database[,apollo_control$indivID]))
+    indivs  <- unique(database[,apollo_control$indivID])
     nIndivs <- length(indivs)
-    if(!silent){
-      cat("\n ", nRep, " new datasets will be constructed by randomly sampling", sep="")
-      cat("\n ", nIndivs, " individuals with replacement from the original dataset.", sep="")
-    }
+    if(!silent) apollo_print(paste0(nRep, " new datasets will be constructed by randomly sampling ",
+                                    nIndivs, " individuals with replacement from the original dataset."))
     tmp <- as.vector(table(database[,apollo_control$indivID])) # n obs per indiv
     tmp <- all(tmp==tmp[1]) # TRUE if all indivs have same number of obs
-    if(!tmp & !silent){
-      cat("\n Not all individuals have the same number of observations,")
-      cat("\n therefore not all generated datasets may have the same")
-      cat("\n number of observations.")
-    }
+    if(!tmp & !silent) apollo_print(paste0("Not all individuals have the same number of observations, ",
+                                           "therefore not all generated datasets may have the same ",
+                                           "number of observations."))
   } else {
     nRep <- ncol(samples)
-    if(!silent){
-      cat("\n ", nRep, " new dataset will be constructed by sampling from the original", sep="")
-      cat("\n dataset, as described by the 'samples' matrix provided.")
-    }
+    if(!silent) apollo_print(paste0(nRep, " new datasets will be constructed by sampling from the original ",
+                                    "dataset, as described by the 'samples' matrix provided."))
     tmp <- colSums(samples)
     tmp <- all(tmp==tmp[1])
-    if(!tmp & !silent) cat("\n Not all samples have the same number of observations.")
+    if(!tmp & !silent) apollo_print("Not all samples have the same number of observations.")
   }
   
   ### Get number of LL components in model and create stacks
-  if(!silent) cat("\n Preparing bootstrap.")
+  if(!silent) apollo_print("Preparing bootstrap.")
   llComponents <- apollo_probabilities(apollo_beta, apollo_inputs, functionality="output")
-  paramStack   <- matrix(0, nrow=nRep, ncol=length(apollo_beta) , dimnames=list(c(), names(apollo_beta)))
+  paramStack   <- matrix(NA, nrow=nRep, ncol=length(apollo_beta) , dimnames=list(c(), names(apollo_beta)))
   llStack      <- matrix(0, nrow=nRep, ncol=length(llComponents), dimnames=list(c(), paste0("LL_", names(llComponents))) )
   nObsStack    <- rep(0,nRep)
   rm(llComponents)
   
   # Check for previous bootstrap params & samples files
-  tmp <- paste0(name, "_bootstrap_params.csv")
-  if(file.exists(tmp)) oldParams <- tryCatch(utils::read.csv(tmp), error=function(e) return(NA)) else oldParams <- NA
+  fileNameParams <- paste(name, "bootstrap_params.csv", sep="_")
+  if(file.exists(fileNameParams)) oldParams <- tryCatch(utils::read.csv(fileNameParams), error=function(e) return(NA)) else oldParams <- NA
   if(!anyNA(oldParams)){
-    if(!silent) cat("\n File ", tmp, "\n  found in the working directory.", sep="")
+    if(!silent) apollo_print(paste0("File ", fileNameParams, " found in the working directory."))
     checkCol  <- colnames(oldParams) == c(colnames(paramStack), colnames(llStack), "obs")
     if(!all(checkCol)){
       oldParams <- NA
-      if(!silent) cat("\n  But will be ignored as its dimensions are\n  incompatible with current model.")
+      if(!silent) apollo_print("But will be ignored as its dimensions are incompatible with current model.")
     }
   }
-  tmp <- paste0(name, "_bootstrap_samples.csv")
-  if(file.exists(tmp)) oldSamples <- tryCatch(utils::read.csv(tmp), error=function(e) return(NA)) else oldSamples <- NA
+  fileNameSamples <- paste0(name, "_bootstrap_samples.csv")
+  if(file.exists(fileNameSamples)) oldSamples <- tryCatch(utils::read.csv(fileNameSamples), error=function(e) return(NA)) else oldSamples <- NA
   if(!anyNA(oldSamples)){
-    if(!silent) cat("\n File ", tmp, "\n  found in the working directory.", sep="")
+    if(!silent) apollo_print(paste0("File ", fileNameSamples, " found in the working directory."))
     checkRow  <- nrow(oldSamples) == nrow(database)
     if(!checkRow){
       oldSamples <- NA
-      if(!silent) cat("\n  But will be ignored as its dimensions are\n  incompatible with current model.")
+      if(!silent) apollo_print("But will be ignored as its dimensions are incompatible with current model.")
     }
   }
-  if(!anyNA(oldParams) & !anyNA(oldSamples)){
-    if(!silent) cat("\n New bootstrap repetitions will be added to the existing results.")
+  if(recycle && !anyNA(oldParams) & !anyNA(oldSamples)){
+    if(!silent) apollo_print("New bootstrap repetitions will be added to the existing results.")
   } else {
     oldParams  <- NA
     oldSamples <- NA
-    if((exists("checkCol") | exists("checkRow")) & !silent) cat("\n Old bootstrap repetitions will be discarded.")
+    if((exists("checkCol") | exists("checkRow")) & !silent){
+      if(recycle) apollo_print("Old bootstrap repetitions will be discarded.")
+      if(!recycle) apollo_print("Old bootstrap repetitions will be discarded, as setting 'recycle' is FALSE.")
+    } 
   }
   
   ### Create samples matrix if necessary
@@ -202,42 +232,36 @@ apollo_bootstrap <- function(apollo_beta, apollo_fixed,
   
   # BOOTSTRAP LOOP
   if(!silent){
-    cat("\n Parameters and LL in each repetition will be written to:")
-    cat("\n  ", paste0(apollo_control$modelName, "_bootstrap_params.csv"), sep="")
-    cat("\n Vectors with the sample in each repetition will be written to:")
-    cat("\n  ", paste0(apollo_control$modelName, "_bootstrap_samples.csv"), sep="")
-    cat("\n")
+    apollo_print(paste0("Parameters and LL in each repetition will be written to: ", fileNameParams))
+    apollo_print(paste0("Vectors showing sampling rate for each observation in each repetition written to: ", fileNameSamples))
+    apollo_print("\n")
   }
   for(i in 1:nRep){
-    newObs    <- rep(1:nrow(database), samples[,i])
-    newObs1=0*newObs
-    j=2
-    while(j<=length(newObs)){
-     if(newObs[j]==newObs[j-1]) newObs1[j]=newObs1[j-1]+1
-    j=j+1}
+    # Create new database and change indivID for repeated indivs
+    newObs  <- rep(1:nrow(database), samples[,i])
+    newObs1 <- 0*newObs
+    for(j in 2:length(newObs)) if(newObs[j]==newObs[j-1]) newObs1[j]=newObs1[j-1]+1
     database2 <- database[newObs,]
     database2[,apollo_control$indivID] <- database2[,apollo_control$indivID]+(newObs1-1)/1000
+    rm(newObs, newObs1)
+    database2 <- database2[order(database2[,apollo_control$indivID]),]
     
+    # Create apollo_inputs2
     apollo_inputs2 <- apollo_validateInputs(database=database2, silent=TRUE)
     apollo_inputs2$apollo_control$noDiagnostics <- TRUE
+    apollo_inputs2$apollo_control$noValidation  <- TRUE
+    apollo_inputs2$silent <- TRUE
     if(apollo_control$mixing) draws <- apollo_makeDraws(apollo_inputs2, silent=TRUE)
     
-    # Initialize cluster if user asked for it
-    if(apollo_control$nCores==1) cl <- NA else {
-      cl <- apollo_makeCluster(apollo_probabilities, apollo_inputs2, silent=TRUE)
-      apollo_control$nCores <- length(cl)
-    }
-    
-    # Create logLike function
-    apollo_logLike <- apollo_makeLogLike(apollo_beta, apollo_fixed, apollo_probabilities,
-                                         apollo_inputs2, estimate_settings, cl)
-    
-    # Estimate
+    # Estimate model
     nObsStack[i] <- nrow(database2)
-    if(!silent)cat("\nEstimation cycle ", i, " (", nObsStack[i], " obs)\n", sep="")
-    model <- maxLik::maxLik(apollo_logLike, start=theta_var_val,
-                            method=estimationRoutine, print.level=estimate_settings$printLevel, finalHessian=FALSE,
-                            iterlim=maxIterations, countIter=FALSE, writeIter=FALSE, sumLL=FALSE)
+    if(!silent) apollo_print(paste0("Estimation cycle ", i, " (", nObsStack[i], " obs)"))
+    estimate_settings$hessianRoutine="none"
+    ### change 28 July
+    estimate_settings$silent=TRUE
+    ### end change
+    model <- apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, 
+                             apollo_inputs2, estimate_settings)
     
     # Check convergence
     succesfulEstimation <- FALSE
@@ -251,7 +275,7 @@ apollo_bootstrap <- function(apollo_beta, apollo_fixed,
     if(succesfulEstimation){
       
       # Closes clusters if using multicore
-      if(exists('cl') & apollo_control$nCores>1) parallel::stopCluster(cl)
+      #if(exists('cl') & apollo_control$nCores>1) parallel::stopCluster(cl)
       
       # Store estimated parameters
       temp <- c(model$estimate, apollo_beta[apollo_fixed])
@@ -259,50 +283,69 @@ apollo_bootstrap <- function(apollo_beta, apollo_fixed,
       paramStack[i,] <- temp
       
       # Store in-sample LL components
-      ll <- apollo_probabilities(c(model$estimate, theta_fix_val), apollo_inputs2, functionality="output")
+      ll <- apollo_probabilities(model$estimate, apollo_inputs2, functionality="output")
       for(j in 1:ncol(llStack)) llStack[i,j] <- ifelse(workInLogs, sum(ll[[j]]), sum(log(ll[[j]])))
       
       # Save results from bootstrap iteration
       temp <- cbind(paramStack[1:i,,drop=FALSE], llStack[1:i,,drop=FALSE], obs=nObsStack[1:i])
-      fileName <- paste(name, "bootstrap_params.csv", sep="_")
       if(!anyNA(oldParams)) temp <- rbind(oldParams, temp)
-      tryCatch(utils::write.csv(temp, fileName, row.names=FALSE),
-               error=function(e) if(!silent) cat(" Could not write to\n ", fileName,"\n"))
+      tryCatch(utils::write.csv(temp, fileNameParams, row.names=FALSE),
+               error=function(e) if(!silent) apollo_print(paste0("Could not write to ", fileNameParams)))
       temp <- samples[,1:i]
       if(!anyNA(oldSamples)) temp <- cbind(oldSamples, temp)
-      fileName <- paste(name, "bootstrap_samples.csv", sep="_")
-      tryCatch(utils::write.csv(temp, fileName, row.names=FALSE), 
-               error=function(e) if(!silent) cat(" Could not write to\n ", fileName))
-      if(!silent) cat("Estimation results written to file.\n")
+      tryCatch(utils::write.csv(temp, fileNameSamples, row.names=FALSE), 
+               error=function(e) if(!silent) apollo_print(paste0("Could not write to ", fileNameSamples)))
+      if(!silent && !calledByEstimate) apollo_print("Estimation results written to file.\n")
     } else {
       # Report error but continue with next iteration
-      if(!silent)cat("\nERROR: Estimation failed in repetition ", i, ".", sep="")
+      if(!silent) apollo_print(paste0("ERROR: Estimation failed in repetition ", i, "."))
       if(estimationRoutine=="bfgs" & !silent) print(as.matrix(round(get("lastFuncParam", envir=globalenv()),4)))
     }
   }
   
   # Calculate covariance matrix, save it and print it
   if(!anyNA(oldParams)) paramStack <- rbind(oldParams[,1:ncol(paramStack),drop=FALSE], paramStack)
-  includeRow <- apply(paramStack, MARGIN=1, function(x) !all(x==0))
+  includeRow <- apply(paramStack, MARGIN=1, function(x) !all(is.na(x)))
   Sigma      <- stats::cov(paramStack[includeRow,])
+  # remove fixed params from Sigma (commented out because apollo_estimate includes the fixed ones)
+  if(length(apollo_fixed)>0){
+  Sigma      <- Sigma[-which(colnames(Sigma) %in% apollo_fixed),-which(colnames(Sigma) %in% apollo_fixed)]
+  if(is.null(rownames(Sigma)) && nrow(Sigma)==ncol(Sigma)) rownames(Sigma)=colnames(Sigma)
+  }
   fileName   <- paste0(name, "_bootstrap_vcov.csv")
-  tryCatch(utils::write.csv(Sigma, fileName, row.names=FALSE),
-           error=function(e) cat("\n Could not write to ", fileName))
+  tryCatch(utils::write.csv(Sigma, fileName, row.names=TRUE),
+           error=function(e) apollo_print(paste0("Could not write to ", fileName)))
   
   if(!silent){
-    cat("\n Parameters and LL in each repetition written to:")
-    cat("\n  ", paste0(name, "_bootstrap_params.csv"), sep="")
-    cat("\n Vectors with the sample in each repetition written to:")
-    cat("\n  ", paste0(name, "_bootstrap_samples.csv"), sep="")
-    cat("\n Covariance matrix of parameters written to\n  ", fileName, sep="")
-    cat("\n")
+    apollo_print(paste0("\n"))
+    apollo_print(paste0("Finished bootstrap runs."))
+    apollo_print(paste0("Parameters and LL for each repetition written to: ", fileNameParams))
+    apollo_print(paste0("Vectors showing sampling rate for each observation in each repetition written to: ", fileNameSamples))
+    apollo_print(paste0("Covariance matrix of parameters written to: ", fileName))
+    apollo_print(paste0("\n"))
+    if(!calledByEstimate){
+    apollo_print(paste0("Mean LL across runs: ", round(mean(llStack[,ncol(llStack)]),2)))
+    apollo_print("\nMean parameter values across runs: ")
+    txt=(data.frame(round(colMeans(paramStack[includeRow,]),4)))
+    colnames(txt)="Estimate"
+    print(txt)
+    apollo_print("\nCovariance matrix across runs:")
+    longNames <- colnames(Sigma)
+    maxLen    <- max(nchar(longNames))
+    for(i in 1:length(longNames)) longNames[i] <- paste0(paste0(rep(" ", maxLen-nchar(longNames[i])), collapse=""), longNames[i])
+    tmp <- Sigma #round(Sigma,4)
+    colnames(tmp) <- longNames
+    if(nrow(tmp)==ncol(tmp)) rownames(tmp) <- longNames
+    print(tmp, digits=4)}
   }
   
   # Stop clock and return results
   endtime   <- Sys.time()
   timeTaken <- difftime(endtime, starttime, units='auto')
-  if(!silent) cat("\nBootstrap processing time: ", format(timeTaken), "\n",sep="")
+  if(!silent) apollo_print(paste0("Bootstrap processing time: ", format(timeTaken)))
   #output_matrix <- cbind(paramStack, llStack, nObs=nObsStack)
-  if(!silent) cat("\nBootstrap covariance matrix produced\n")
-  return(Sigma)
+  #if(!silent) apollo_print("Bootstrap covariance matrix produced")
+  return(invisible(list(estimates=paramStack[includeRow,],
+                        varcov=Sigma,
+                        LL=llStack[,ncol(llStack)])))
 }
