@@ -33,31 +33,30 @@ apollo_sharesTest=function(model, apollo_probabilities, apollo_inputs, sharesTes
   subsamples   = sharesTest_settings[["subsamples"]]
   
   predictedShares = apollo_prediction(model, apollo_probabilities, apollo_inputs, 
-                                      prediction_settings=list(modelComponent=sharesTest_settings$modelComponent,silent=TRUE))
+                                      prediction_settings=list(modelComponent=sharesTest_settings$modelComponent,silent=TRUE))[[1]]
   
   #### predictedShares=predictedShares[,-ncol(predictedShares)] ### removed
-  predictedShares=predictedShares[,!colnames(predictedShares)%in%c("ID","Choice situation","chosen")]#### NEW
+  predictedShares=predictedShares[,!colnames(predictedShares)%in%c("ID","Observation","chosen")]#### NEW
   
-  ### If there are any NA in predicted Shares, then it assumes it comes from rows
+  ### SH new lines 4/4. Sort out full sample here
+  if(length(subsamples)==1 && is.na(subsamples)) subsamples=list("All data"=rep(1,length(choiceVar)))
+  
+  ### If there are any NA in predicted Shares, then it assumes these come from rows
   ### and removes them from the analysis
   if(anyNA(predictedShares)){
     rows <- !is.na(rowSums(predictedShares))
     choiceVar <- choiceVar[rows]
     subsamples <- lapply(subsamples, function(x) x[rows])
     predictedShares <- predictedShares[rows,]
-    cat("\nWarning: Predicted values contain NA.\n These rows will be ommited from the analysis.")
+    apollo_print("Warning: Predicted values contain NA. This could be due to using the rows setting in the model. These observations will be ommited from the analysis.")
   }
   
-  
-  
   xnames=colnames(predictedShares)
   predictedShares=split(predictedShares, rep(1:ncol(predictedShares), each = nrow(predictedShares)))
   names(predictedShares)=xnames
   
-  
-  
   categories = subsamples
-  
+
   ### Check that values in 'categories' are either 0/1 or boolean
   isValid <- function(x){
     ux <- unique(x)
@@ -75,25 +74,23 @@ apollo_sharesTest=function(model, apollo_probabilities, apollo_inputs, sharesTes
   trueShares = list()
   for(i in 1:length(alternatives)) trueShares[[names(alternatives)[i]]] <- (choiceVar==alternatives[i])
   
-  if(anyNA(categories)){
-    categories=list()
-    categories[["All data"]]=rep(1,length(trueShares[[1]]))
-  } else {
+  # SH changes 4/4/20
+  #if(anyNA(categories)){
+  #  categories=list()
+  #  categories[["All data"]]=rep(1,length(trueShares[[1]]))
+  #} else {
     if(!all(names(alternatives) %in% names(predictedShares))) stop("\nPredicted choice probabilities should be provided for all alternatives.")
     if(any(lapply(categories,sum)==0)) stop("\nSome subsamples are empty!)")
     if(!any(lapply(categories,sum)==length(trueShares[[1]])) & !any(lapply(categories,length)==1)) categories[["All data"]]=rep(1,length(trueShares[[1]]))
-  }
+  #}
   
   cat("\nRunning share prediction tests\n")
-  cat("\n")
   iterations=length(categories)
-  j=1
-  while(j<=iterations){
+  for(j in 1:iterations){
     output=matrix(0,nrow=6,ncol=length(trueShares)+1)  
     colnames(output)=c(names(trueShares),"All")
     rownames(output)=c("Times chosen (data)","Times chosen (prediction)","SD prediction","Diff (prediction-data)","t-ratio","p-val")
-    k=1
-    while(k<=length(trueShares)){
+    for(k in 1:length(trueShares)){
       temp1=as.data.frame(cbind(trueShares[[k]],categories[[j]]))
       temp2=as.data.frame(cbind(predictedShares[[k]],categories[[j]]))
       output[1,k]=colSums(temp1[temp1$V2==1,])[1]
@@ -107,23 +104,21 @@ apollo_sharesTest=function(model, apollo_probabilities, apollo_inputs, sharesTes
         output[5,k]=(output[2,k]-output[1,k])/output[3,k]
         output[6,k]=round(2*(1-stats::pnorm(abs(output[5,k]))),3)  
       }
-      k=k+1
     }
-    output[1,k]=sum(output[1,(1:(k-1))])
-    output[2,k]=sum(output[2,(1:(k-1))])
-    output[3,k]=NA
-    output[4,k]=sum(output[4,(1:(k-1))])
-    output[5,k]=NA
-    output[6,k]=NA
-    cat("\nPrediction tests for group: ",names(categories)[[j]]," (",sum(categories[[j]])," observations)",sep="")
+    output[1,(length(trueShares)+1)]=sum(output[1,(1:length(trueShares))])
+    output[2,(length(trueShares)+1)]=sum(output[2,(1:length(trueShares))])
+    output[3,(length(trueShares)+1)]=NA
+    output[4,(length(trueShares)+1)]=sum(output[4,(1:length(trueShares))])
+    output[5,(length(trueShares)+1)]=NA
+    output[6,(length(trueShares)+1)]=NA
+    cat("\nPrediction test for group: ",names(categories)[[j]]," (",sum(categories[[j]])," observations)",sep="")
     cat("\n")
     cat("\n")
     output=output[-3,]
     print(round(output,3))
-    if(output[1,k]!=sum(categories[[j]])){
+    if(output[1,k+1]!=sum(categories[[j]])){
       cat("\nWarning: the totals in the final column are not equal to the number of observations!")
       cat("\nThis is normal if you're working with only a subset of possible alternatives in the columns.\n")
     }
-    j=j+1
   }
 }
